@@ -8,7 +8,7 @@ fn main() -> Result<()> {
     stdin.read_to_string(&mut buffer)?;
     let basedir = std::path::Path::new(".");
     let style = Style::default();
-    let mut doc = Document::from_json(&basedir, vec![], &buffer, style)?;
+    let mut doc = Document::from_json(&basedir, vec![], &buffer, style, None)?;
     doc.typeset();
     let bytes = doc.ast()?.into_bytes();
     io::stdout().write_all(&bytes)?;
