@@ -16,8 +16,8 @@ use std::str::FromStr;
 use std::sync::Arc;
 
 use lazy_static::lazy_static;
+use log::trace;
 use regex::{escape, Regex, RegexBuilder};
-use tracing::{event, instrument, Level};
 
 #[derive(Debug, PartialEq, Eq, Hash, Copy, Clone, Serialize, Deserialize)]
 #[serde(rename_all = "lowercase")]
@@ -393,17 +393,11 @@ mod test_binding {
 }
 
 /// Set of all known bindings.
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub struct Bindings {
     bindings: Vec<Binding>,
 }
 
-impl Default for Bindings {
-    fn default() -> Self {
-        Bindings { bindings: vec![] }
-    }
-}
-
 #[derive(Debug, Deserialize)]
 struct ParsedImpl {
     function: String,
@@ -468,7 +462,6 @@ impl Bindings {
     }
 
     /// Add bindings from a YAML string
-    #[instrument(level = "trace", skip(self, yaml))]
     pub fn add_from_yaml(&mut self, yaml: &str) -> Result<()> {
         let bindings: Vec<ParsedBindingWrapper> = serde_yaml::from_str(yaml)?;
         for wrapper in bindings {
@@ -506,14 +499,13 @@ impl Bindings {
     }
 
     /// Add bindings from a file.
-    #[instrument(level = "trace", skip(self))]
     pub fn add_from_file<P>(&mut self, filename: P, template: Option<&str>) -> Result<()>
     where
         P: AsRef<Path> + Debug,
     {
         let yaml = resource::read_as_string(filename.as_ref(), template)
             .map_err(|e| SubplotError::BindingsFileNotFound(filename.as_ref().into(), e))?;
-        event!(Level::TRACE, "Loaded file content");
+        trace!("Loaded file content");
         self.add_from_yaml(&yaml).map_err(|e| {
             SubplotError::BindingFileParseError(filename.as_ref().to_owned(), Box::new(e))
         })?;
@@ -530,7 +522,6 @@ impl Bindings {
     }
 }
 
-#[instrument(level = "trace", skip(parsed))]
 fn from_hashmap(parsed: &ParsedBinding) -> Result<Binding> {
     let given: i32 = parsed.given.is_some().into();
     let when: i32 = parsed.when.is_some().into();
@@ -566,16 +557,10 @@ fn from_hashmap(parsed: &ParsedBinding) -> Result<Binding> {
         regex_from_simple_pattern(pattern, parsed.regex.is_some(), &mut types)?
     };
 
-    event!(
-        Level::TRACE,
-        ?kind,
-        ?pattern,
-        ?types,
-        "Successfully acquired binding"
-    );
+    trace!("Successfully acquired binding");
 
     let mut ret = Binding::new(kind, &pattern, parsed.case_sensitive, types)?;
-    event!(Level::TRACE, "Binding parsed OK");
+    trace!("Binding parsed OK");
     for (template, pimpl) in &parsed.impls {
         ret.add_impl(template, &pimpl.function, pimpl.cleanup.as_deref());
     }
