@@ -9,28 +9,19 @@ use base64::encode;
 use serde::Serialize;
 use tera::{Context, Tera, Value};
 
-/// Return the requested template specification.
-pub fn template_spec(doc: &Document) -> Result<TemplateSpec, SubplotError> {
-    let template = doc
-        .meta()
-        .template_name()
-        .ok_or(SubplotError::MissingTemplate)?;
-
-    let mut filename = PathBuf::from(template);
-    filename.push("template");
-    filename.push("template.yaml");
-    TemplateSpec::from_file(&filename)
-}
-
 /// Generate a test program from a document, using a template spec.
 pub fn generate_test_program(
     doc: &mut Document,
-    spec: &TemplateSpec,
     filename: &Path,
     template: &str,
 ) -> Result<(), SubplotError> {
     let context = context(doc, template)?;
-    let code = tera(spec, template)?
+    let docimpl = doc
+        .meta()
+        .document_impl(template)
+        .ok_or(SubplotError::MissingTemplate)?;
+
+    let code = tera(docimpl.spec(), template)?
         .render("template", &context)
         .expect("render");
     write(filename, &code)?;
@@ -43,12 +34,13 @@ fn context(doc: &mut Document, template: &str) -> Result<Context, SubplotError>
     context.insert("scenarios", &scenarios);
     context.insert("files", doc.files());
 
-    let funcs_filenames = doc.meta().functions_filenames();
     let mut funcs = vec![];
-    for filename in funcs_filenames {
-        let content = resource::read_as_string(filename, Some(template))
-            .map_err(|err| SubplotError::FunctionsFileNotFound(filename.into(), err))?;
-        funcs.push(Func::new(filename, content));
+    if let Some(docimpl) = doc.meta().document_impl(template) {
+        for filename in docimpl.functions_filenames() {
+            let content = resource::read_as_string(filename, Some(template))
+                .map_err(|err| SubplotError::FunctionsFileNotFound(filename.into(), err))?;
+            funcs.push(Func::new(filename, content));
+        }
     }
     context.insert("functions", &funcs);
 
