@@ -12,8 +12,8 @@ use crate::Scenario;
 use crate::ScenarioStep;
 use crate::Style;
 use crate::{bindings::CaptureType, parser::parse_scenario_snippet};
-use crate::{template_spec, TemplateSpec};
 use crate::{Result, SubplotError};
+use crate::{Warning, Warnings};
 
 use std::collections::HashSet;
 use std::default::Default;
@@ -24,7 +24,7 @@ use std::str::FromStr;
 
 use pandoc_ast::{MutVisitor, Pandoc};
 
-use tracing::{event, instrument, span, Level};
+use log::{error, trace};
 
 /// The set of known (special) classes which subplot will always recognise
 /// as being valid.
@@ -74,7 +74,7 @@ static KNOWN_PANDOC_CLASSES: &[&str] = &["numberLines", "noNumberLines"];
 /// use subplot;
 /// let basedir = std::path::Path::new(".");
 /// let style = subplot::Style::default();
-/// let doc = subplot::Document::from_file(&basedir, filename, style).unwrap();
+/// let doc = subplot::Document::from_file(&basedir, filename, style, None).unwrap();
 /// assert_eq!(doc.files(), &[]);
 /// ~~~~
 #[derive(Debug)]
@@ -84,6 +84,7 @@ pub struct Document {
     meta: Metadata,
     files: DataFiles,
     style: Style,
+    warnings: Warnings,
 }
 
 impl<'a> Document {
@@ -100,22 +101,28 @@ impl<'a> Document {
             meta,
             files,
             style,
+            warnings: Warnings::default(),
         }
     }
 
-    #[instrument(level = "trace", skip(ast))]
+    /// Return all warnings about this document.
+    pub fn warnings(&self) -> &[Warning] {
+        self.warnings.warnings()
+    }
+
     fn from_ast<P>(
         basedir: P,
         markdowns: Vec<PathBuf>,
         mut ast: Pandoc,
         style: Style,
+        template: Option<&str>,
     ) -> Result<Document>
     where
         P: AsRef<Path> + Debug,
     {
-        let meta = Metadata::new(basedir, &ast)?;
+        let meta = Metadata::new(basedir, &ast, template)?;
         let mut linter = LintingVisitor::default();
-        event!(Level::TRACE, "Walking AST for linting...");
+        trace!("Walking AST for linting...");
         linter.walk_pandoc(&mut ast);
         if !linter.issues.is_empty() {
             // Currently we can't really return more than one error so return one
@@ -123,24 +130,24 @@ impl<'a> Document {
         }
         let files = DataFiles::new(&mut ast);
         let doc = Document::new(markdowns, ast, meta, files, style);
-        event!(Level::TRACE, "Loaded from JSON OK");
+        trace!("Loaded from JSON OK");
         Ok(doc)
     }
 
     /// Construct a Document from a JSON AST
-    #[instrument(level = "trace", skip(json))]
     pub fn from_json<P>(
         basedir: P,
         markdowns: Vec<PathBuf>,
         json: &str,
         style: Style,
+        template: Option<&str>,
     ) -> Result<Document>
     where
         P: AsRef<Path> + Debug,
     {
-        event!(Level::TRACE, "Parsing document...");
+        trace!("Parsing document...");
         let ast: Pandoc = serde_json::from_str(json)?;
-        Self::from_ast(basedir, markdowns, ast, style)
+        Self::from_ast(basedir, markdowns, ast, style, template)
     }
 
     /// Construct a Document from a named file.
@@ -148,8 +155,17 @@ impl<'a> Document {
     /// The file can be in any format Pandoc understands. This runs
     /// Pandoc to parse the file into an AST, so it can be a little
     /// slow.
-    #[instrument(level = "trace")]
-    pub fn from_file(basedir: &Path, filename: &Path, style: Style) -> Result<Document> {
+    pub fn from_file(
+        basedir: &Path,
+        filename: &Path,
+        style: Style,
+        template: Option<&str>,
+    ) -> Result<Document> {
+        trace!(
+            "Document::from_file: basedir={} filename={}",
+            basedir.display(),
+            filename.display()
+        );
         let markdowns = vec![filename.to_path_buf()];
 
         let mut pandoc = pandoc::new();
@@ -164,13 +180,14 @@ impl<'a> Document {
         // Add external Pandoc filters.
         crate::policy::add_citeproc(&mut pandoc);
 
-        event!(Level::TRACE, ?filename, "Invoking Pandoc to parse document");
+        trace!("Invoking Pandoc to parse document {:?}", filename);
         let output = match pandoc.execute()? {
             pandoc::PandocOutput::ToBuffer(o) => o,
             _ => return Err(SubplotError::NotJson),
         };
-        let doc = Document::from_json(basedir, markdowns, &output, style)?;
-        event!(Level::TRACE, "Loaded document OK");
+        trace!("Pandoc was happy");
+        let doc = Document::from_json(basedir, markdowns, &output, style, template)?;
+        trace!("Loaded document OK");
         Ok(doc)
     }
 
@@ -179,23 +196,19 @@ impl<'a> Document {
     /// The file can be in the CommonMark format, with some
     /// extensions. This uses the pulldown-cmark crate to parse the
     /// file into an AST.
-    #[instrument(level = "trace")]
     pub fn from_file_with_pullmark(
         basedir: &Path,
         filename: &Path,
         style: Style,
+        template: Option<&str>,
     ) -> Result<Document> {
-        event!(
-            Level::TRACE,
-            ?filename,
-            "Parsing document with pullmark-cmark"
-        );
+        trace!("Parsing document with pullmark-cmark from {:?}", filename);
         let filename = filename.to_path_buf();
         let markdown = std::fs::read_to_string(&filename)?;
         let ast = ast::AbstractSyntaxTree::from_str(&markdown)?;
 
-        event!(Level::TRACE, "Parsed document OK");
-        Self::from_ast(basedir, vec![filename], ast.to_pandoc(), style)
+        trace!("Parsed document OK");
+        Self::from_ast(basedir, vec![filename], ast.to_pandoc(), style, template)
     }
 
     /// Return the AST of a Document, serialized as JSON.
@@ -216,15 +229,27 @@ impl<'a> Document {
     ///
     /// The sources are any files that affect the output so that if
     /// the source file changes, the output needs to be re-generated.
-    pub fn sources(&mut self) -> Vec<PathBuf> {
+    pub fn sources(&mut self, template: Option<&str>) -> Vec<PathBuf> {
         let mut names = vec![];
 
         for x in self.meta().bindings_filenames() {
             names.push(PathBuf::from(x))
         }
 
-        for x in self.meta().functions_filenames() {
-            names.push(PathBuf::from(x))
+        if let Some(template) = template {
+            if let Some(spec) = self.meta().document_impl(template) {
+                for x in spec.functions_filenames() {
+                    names.push(PathBuf::from(x));
+                }
+            }
+        } else {
+            for template in self.meta().templates() {
+                if let Some(spec) = self.meta().document_impl(template) {
+                    for x in spec.functions_filenames() {
+                        names.push(PathBuf::from(x));
+                    }
+                }
+            }
         }
 
         for x in self.meta().bibliographies().iter() {
@@ -250,13 +275,12 @@ impl<'a> Document {
     }
 
     /// Check the document for common problems.
-    #[instrument(level = "trace", skip(self))]
     pub fn lint(&self) -> Result<()> {
-        event!(Level::TRACE, "Linting document");
+        trace!("Linting document");
         self.check_doc_has_title()?;
         self.check_filenames_are_unique()?;
         self.check_block_classes()?;
-        event!(Level::TRACE, "No linting problems found");
+        trace!("No linting problems found");
         Ok(())
     }
 
@@ -315,14 +339,13 @@ impl<'a> Document {
 
     /// Check that all named files (in matched steps) are actually present in the
     /// document.
-    #[instrument(level = "trace", skip(self))]
     pub fn check_named_files_exist(&mut self, template: &str) -> Result<bool> {
         let filenames: HashSet<_> = self
             .files()
             .iter()
             .map(|f| f.filename().to_lowercase())
             .collect();
-        event!(Level::TRACE, ?filenames, "Checking that files exist");
+        trace!("Checking that files exist");
         let mut okay = true;
         let scenarios = match self.matched_scenarios(template) {
             Ok(scenarios) => scenarios,
@@ -335,7 +358,10 @@ impl<'a> Document {
                         if matches!(step.types().get(name.as_str()), Some(CaptureType::File))
                             && !filenames.contains(&text.to_lowercase())
                         {
-                            eprintln!("Found reference to unknown file {}", text);
+                            self.warnings.push(Warning::UnknownEmbeddedFile(
+                                scenario.title().to_string(),
+                                text.to_string(),
+                            ));
                             okay = false;
                         }
                     }
@@ -345,36 +371,59 @@ impl<'a> Document {
         Ok(okay)
     }
 
+    /// Check that all embedded files are used by matched steps.
+    pub fn check_embedded_files_are_used(&mut self, template: &str) -> Result<bool> {
+        let mut filenames: HashSet<_> = self
+            .files()
+            .iter()
+            .map(|f| f.filename().to_lowercase())
+            .collect();
+        trace!("Checking that files are used");
+        let scenarios = match self.matched_scenarios(template) {
+            Ok(scenarios) => scenarios,
+            Err(_) => return Ok(true), // We can't do the check, so say it's okay.
+        };
+        for scenario in scenarios {
+            for step in scenario.steps() {
+                for captured in step.parts() {
+                    if let PartialStep::CapturedText { name, text } = captured {
+                        if matches!(step.types().get(name.as_str()), Some(CaptureType::File)) {
+                            filenames.remove(&text.to_lowercase());
+                        }
+                    }
+                }
+            }
+        }
+        for filename in filenames.iter() {
+            self.warnings
+                .push(Warning::UnusedEmbeddedFile(filename.to_string()));
+        }
+
+        // We always succeed. Subplot's own subplot had valid cases of
+        // an embedded file being used and we need to develop a way to
+        // mark such uses as OK, before we can make it an error to not
+        // use an embedded file in a scenario.
+        Ok(true)
+    }
+
     /// Check that all matched steps actually have function implementations
-    #[instrument(level = "trace", skip(self))]
     pub fn check_matched_steps_have_impl(&mut self, template: &str) -> bool {
-        event!(Level::TRACE, "Checking that steps have implementations");
+        trace!("Checking that steps have implementations");
         let mut okay = true;
         let scenarios = match self.matched_scenarios(template) {
             Ok(scenarios) => scenarios,
             Err(_) => return true, // No matches means no missing impls
         };
-        event!(Level::TRACE, count = scenarios.len(), "Found scenarios");
+        trace!("Found {} scenarios", scenarios.len());
         for scenario in scenarios {
-            event!(
-                Level::TRACE,
-                title = scenario.title(),
-                "Checking that steps in scenario"
-            );
-            let mut said_scenario = false;
+            trace!("Checking that steps in scenario");
             for step in scenario.steps() {
                 if step.function().is_none() {
-                    if !said_scenario {
-                        eprintln!("Scenario: '{}'", scenario.title());
-                        eprintln!("  Template: '{}'", template);
-                        said_scenario = true;
-                    }
-                    eprintln!("  Step missing implementation: '{}'", step.text());
-                    event!(
-                        Level::ERROR,
-                        step = step.text(),
-                        "Missing step implementation"
-                    );
+                    trace!("Missing step implementation: {:?}", step.text());
+                    self.warnings.push(Warning::MissingStepImplementation(
+                        scenario.title().to_string(),
+                        step.text().to_string(),
+                    ));
                     okay = false;
                 }
             }
@@ -387,6 +436,7 @@ impl<'a> Document {
         let mut visitor =
             visitor::TypesettingVisitor::new(self.style.clone(), self.meta.bindings());
         visitor.walk_pandoc(&mut self.ast);
+        self.warnings.push_all(visitor.warnings());
     }
 
     /// Return all scenarios in a document.
@@ -408,11 +458,9 @@ impl<'a> Document {
     }
 
     /// Return matched scenarios in a document.
-    #[instrument(skip(self))]
     pub fn matched_scenarios(&mut self, template: &str) -> Result<Vec<MatchedScenario>> {
         let scenarios = self.scenarios()?;
-        event!(
-            Level::TRACE,
+        trace!(
             "Found {} scenarios, checking their bindings",
             scenarios.len()
         );
@@ -422,29 +470,37 @@ impl<'a> Document {
             .map(|scen| MatchedScenario::new(template, scen, bindings))
             .collect()
     }
+
+    /// Extract a template name from this document
+    pub fn template(&self) -> Result<&str> {
+        let templates: Vec<_> = self.meta().templates().collect();
+        if templates.len() == 1 {
+            Ok(templates[0])
+        } else if templates.is_empty() {
+            Err(SubplotError::MissingTemplate)
+        } else {
+            Err(SubplotError::AmbiguousTemplate)
+        }
+    }
 }
 
 /// Load a `Document` from a file.
 ///
 /// This version uses Pandoc to parse the Markdown.
-#[instrument(level = "trace")]
-pub fn load_document<P>(filename: P, style: Style) -> Result<Document>
+pub fn load_document<P>(filename: P, style: Style, template: Option<&str>) -> Result<Document>
 where
     P: AsRef<Path> + Debug,
 {
     let filename = filename.as_ref();
     let base_path = get_basedir_from(filename);
-    event!(
-        Level::TRACE,
-        ?filename,
-        ?base_path,
+    trace!(
         "Loading document based at `{}` called `{}` with {:?}",
         base_path.display(),
         filename.display(),
         style
     );
-    let doc = Document::from_file(&base_path, filename, style)?;
-    event!(Level::TRACE, "Loaded doc from file OK");
+    let doc = Document::from_file(&base_path, filename, style, template)?;
+    trace!("Loaded doc from file OK");
 
     Ok(doc)
 }
@@ -452,63 +508,70 @@ where
 /// Load a `Document` from a file.
 ///
 /// This version uses the `cmark-pullmark` crate to parse Markdown.
-#[instrument(level = "trace")]
-pub fn load_document_with_pullmark<P>(filename: P, style: Style) -> Result<Document>
+pub fn load_document_with_pullmark<P>(
+    filename: P,
+    style: Style,
+    template: Option<&str>,
+) -> Result<Document>
 where
     P: AsRef<Path> + Debug,
 {
     let filename = filename.as_ref();
     let base_path = get_basedir_from(filename);
-    event!(
-        Level::TRACE,
-        ?filename,
-        ?base_path,
+    trace!(
         "Loading document based at `{}` called `{}` with {:?} using pullmark-cmark",
         base_path.display(),
         filename.display(),
         style
     );
     crate::resource::add_search_path(filename.parent().unwrap());
-    let doc = Document::from_file_with_pullmark(&base_path, filename, style)?;
-    event!(Level::TRACE, "Loaded doc from file OK");
+    let doc = Document::from_file_with_pullmark(&base_path, filename, style, template)?;
+    trace!("Loaded doc from file OK");
     Ok(doc)
 }
 
 /// Generate code for one document.
-pub fn codegen(filename: &Path, output: &Path) -> Result<CodegenOutput> {
-    let span = span!(Level::TRACE, "codegen");
-    let _enter = span.enter();
-
-    let mut doc = load_document_with_pullmark(filename, Style::default())?;
+pub fn codegen(filename: &Path, output: &Path, template: Option<&str>) -> Result<CodegenOutput> {
+    let r = load_document_with_pullmark(filename, Style::default(), template);
+    let mut doc = match r {
+        Ok(doc) => doc,
+        Err(err) => {
+            return Err(err);
+        }
+    };
     doc.lint()?;
-    let template = doc
-        .meta()
-        .template_name()
-        .ok_or(SubplotError::MissingTemplate)?
+    let template = template
+        .map(Ok)
+        .unwrap_or_else(|| doc.template())?
         .to_string();
-    event!(Level::TRACE, ?template);
-    if !doc.check_named_files_exist(&template)? || !doc.check_matched_steps_have_impl(&template) {
-        event!(Level::ERROR, "Found problems in document, cannot continue");
-        eprintln!("Unable to continue");
+    trace!("Template: {:?}", template);
+    if !doc.meta().templates().any(|t| t == template) {
+        return Err(SubplotError::TemplateSupportNotPresent);
+    }
+    if !doc.check_named_files_exist(&template)?
+        || !doc.check_matched_steps_have_impl(&template)
+        || !doc.check_embedded_files_are_used(&template)?
+    {
+        error!("Found problems in document, cannot continue");
         std::process::exit(1);
     }
 
-    event!(Level::TRACE, "Generating code");
-    let spec = template_spec(&doc)?;
-    generate_test_program(&mut doc, &spec, output, &template)?;
-    event!(Level::TRACE, "Finished generating code");
+    trace!("Generating code");
 
-    Ok(CodegenOutput::new(spec, doc))
+    generate_test_program(&mut doc, output, &template)?;
+    trace!("Finished generating code");
+
+    Ok(CodegenOutput::new(template, doc))
 }
 
 pub struct CodegenOutput {
-    pub spec: TemplateSpec,
+    pub template: String,
     pub doc: Document,
 }
 
 impl CodegenOutput {
-    fn new(spec: TemplateSpec, doc: Document) -> Self {
-        Self { spec, doc }
+    fn new(template: String, doc: Document) -> Self {
+        Self { template, doc }
     }
 }
 
@@ -573,7 +636,6 @@ mod test_extract {
     }
 
     fn check_result(r: Result<(Option<Scenario>, usize)>, title: Option<&str>, i: usize) {
-        eprintln!("checking result: {:?}", r);
         assert!(r.is_ok());
         let (actual_scen, actual_i) = r.unwrap();
         if title.is_none() {
