@@ -17,11 +17,11 @@ pub struct MatchedScenario {
 
 impl MatchedScenario {
     /// Construct a new matched scenario
-    pub fn new(scen: &Scenario, bindings: &Bindings) -> Result<MatchedScenario> {
+    pub fn new(template: &str, scen: &Scenario, bindings: &Bindings) -> Result<MatchedScenario> {
         let steps: Result<Vec<MatchedStep>> = scen
             .steps()
             .iter()
-            .map(|step| bindings.find(step))
+            .map(|step| bindings.find(template, step))
             .collect();
         Ok(MatchedScenario {
             title: scen.title().to_string(),
@@ -33,6 +33,11 @@ impl MatchedScenario {
     pub fn steps(&self) -> &[MatchedStep] {
         &self.steps
     }
+
+    /// Retrieve the scenario title
+    pub fn title(&self) -> &str {
+        &self.title
+    }
 }
 
 /// A list of matched steps.
@@ -50,11 +55,7 @@ impl MatchedSteps {
 
 impl std::fmt::Display for MatchedSteps {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::result::Result<(), std::fmt::Error> {
-        let matches: Vec<String> = self
-            .matches
-            .iter()
-            .map(|m| format!("{}", m.pattern()))
-            .collect();
+        let matches: Vec<String> = self.matches.iter().map(|m| m.pattern()).collect();
         write!(f, "{}", matches.join("\n"))
     }
 }
@@ -69,22 +70,23 @@ pub struct MatchedStep {
     pattern: String,
     text: String,
     parts: Vec<PartialStep>,
-    function: String,
+    function: Option<String>,
     cleanup: Option<String>,
     types: HashMap<String, CaptureType>,
 }
 
 impl MatchedStep {
     /// Return a new empty match. Empty means it has no step parts.
-    pub fn new(binding: &Binding, types: &HashMap<String, CaptureType>) -> MatchedStep {
+    pub fn new(binding: &Binding, template: &str) -> MatchedStep {
+        let bimpl = binding.step_impl(template);
         MatchedStep {
             kind: binding.kind(),
             pattern: binding.pattern().to_string(),
             text: "".to_string(),
             parts: vec![],
-            function: binding.function().to_string(),
-            cleanup: binding.cleanup().map(String::from),
-            types: types.clone(),
+            function: bimpl.clone().map(|b| b.function().to_owned()),
+            cleanup: bimpl.and_then(|b| b.cleanup().map(String::from)),
+            types: binding.types().map(|(s, c)| (s.to_string(), c)).collect(),
         }
     }
 
@@ -94,14 +96,14 @@ impl MatchedStep {
     }
 
     /// The name of the function to call for the step.
-    pub fn function(&self) -> &str {
-        &self.function
+    pub fn function(&self) -> Option<&str> {
+        self.function.as_deref()
     }
 
     /// Append a partial step to the match.
     pub fn append_part(&mut self, part: PartialStep) {
         self.parts.push(part);
-        self.text = self.text();
+        self.text = self.update_text();
     }
 
     /// Iterate over all partial steps in the match.
@@ -113,7 +115,7 @@ impl MatchedStep {
         self.pattern.to_string()
     }
 
-    fn text(&self) -> String {
+    fn update_text(&self) -> String {
         let mut t = String::new();
         for part in self.parts() {
             t.push_str(part.as_text());
@@ -121,6 +123,11 @@ impl MatchedStep {
         t
     }
 
+    /// Return the step's text
+    pub fn text(&self) -> &str {
+        &self.text
+    }
+
     /// Return the typemap for the matched step
     pub fn types(&self) -> &HashMap<String, CaptureType> {
         &self.types
@@ -160,7 +167,7 @@ impl PartialStep {
     fn as_text(&self) -> &str {
         match self {
             PartialStep::UncapturedText(snippet) => snippet.text(),
-            PartialStep::CapturedText { text, .. } => &text,
+            PartialStep::CapturedText { text, .. } => text,
         }
     }
 }
