@@ -1,13 +1,14 @@
 use crate::Result;
 use crate::{Bindings, TemplateSpec};
 
+use std::collections::HashMap;
 use std::fmt::Debug;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
 
 use pandoc_ast::{Inline, Map, MetaValue, Pandoc};
 
-use tracing::{event, instrument, Level};
+use log::trace;
 
 /// Metadata of a document, as needed by Subplot.
 #[derive(Debug)]
@@ -16,55 +17,63 @@ pub struct Metadata {
     date: Option<String>,
     bindings_filenames: Vec<PathBuf>,
     bindings: Bindings,
-    functions_filenames: Vec<PathBuf>,
-    template: Option<String>,
-    spec: Option<TemplateSpec>,
+    impls: HashMap<String, DocumentImpl>,
     bibliographies: Vec<PathBuf>,
     /// Extra class names which should be considered 'correct' for this document
     classes: Vec<String>,
 }
 
+#[derive(Debug)]
+pub struct DocumentImpl {
+    spec: TemplateSpec,
+    functions: Vec<PathBuf>,
+}
+
 impl Metadata {
     /// Construct a Metadata from a Document, if possible.
-    #[instrument(level = "trace", skip(doc))]
-    pub fn new<P>(basedir: P, doc: &Pandoc) -> Result<Metadata>
+    pub fn new<P>(basedir: P, doc: &Pandoc, template: Option<&str>) -> Result<Metadata>
     where
         P: AsRef<Path> + Debug,
     {
         let title = get_title(&doc.meta);
         let date = get_date(&doc.meta);
         let bindings_filenames = get_bindings_filenames(&doc.meta);
-        let functions_filenames = get_functions_filenames(&doc.meta);
         let bibliographies = get_bibliographies(basedir.as_ref(), &doc.meta);
         let classes = get_classes(&doc.meta);
-        event!(
-            Level::TRACE,
-            ?title,
-            ?date,
-            ?bindings_filenames,
-            ?functions_filenames,
-            ?bibliographies,
-            ?classes,
-            "Loaded basic metadata"
-        );
-        let (template, spec) = if let Some((template, spec)) = get_template_spec(&doc.meta)? {
-            (Some(template), Some(spec))
-        } else {
-            (None, None)
-        };
-        event!(Level::TRACE, ?template, ?spec, "Loaded template spec");
+        trace!("Loaded basic metadata");
+
+        let mut impls = HashMap::new();
+
+        if let Some(raw_impls) = doc.meta.get("impls") {
+            match raw_impls {
+                MetaValue::MetaMap(raw_impls) => {
+                    for (impl_name, functions_filenames) in raw_impls.iter() {
+                        let template_spec = load_template_spec(impl_name)?;
+                        let filenames = pathbufs("", functions_filenames);
+                        let docimpl = DocumentImpl::new(template_spec, filenames);
+                        impls.insert(impl_name.to_string(), docimpl);
+                    }
+                }
+                _ => {
+                    trace!("Ignoring unknown raw implementation value");
+                }
+            }
+        }
+
+        let template = template.or_else(|| impls.keys().next().map(String::as_str));
+
         let mut bindings = Bindings::new();
 
-        get_bindings(&bindings_filenames, &mut bindings, template.as_deref())?;
-        event!(Level::TRACE, "Loaded all metadata successfully");
+        get_bindings(&bindings_filenames, &mut bindings, template)?;
+
+        trace!("Loaded all metadata successfully");
+
         Ok(Metadata {
             title,
             date,
             bindings_filenames,
             bindings,
-            functions_filenames,
-            template,
-            spec,
+            impls,
             bibliographies,
             classes,
         })
@@ -85,17 +94,14 @@ impl Metadata {
         self.bindings_filenames.iter().map(|f| f.as_ref()).collect()
     }
 
-    /// Return filename where functions are specified.
-    pub fn functions_filenames(&self) -> Vec<&Path> {
-        self.functions_filenames
-            .iter()
-            .map(|f| f.as_ref())
-            .collect()
+    /// Return the document implementation (filenames, spec, etc) for the given template name
+    pub fn document_impl(&self, template: &str) -> Option<&DocumentImpl> {
+        self.impls.get(template)
     }
 
-    /// Return the name of the code template, if specified.
-    pub fn template_name(&self) -> Option<&str> {
-        self.template.as_deref()
+    /// Return the templates the document expects to implement
+    pub fn templates(&self) -> impl Iterator<Item = &str> {
+        self.impls.keys().map(String::as_str)
     }
 
     /// Return the bindings.
@@ -114,6 +120,20 @@ impl Metadata {
     }
 }
 
+impl DocumentImpl {
+    fn new(spec: TemplateSpec, functions: Vec<PathBuf>) -> Self {
+        Self { spec, functions }
+    }
+
+    pub fn functions_filenames(&self) -> impl Iterator<Item = &Path> {
+        self.functions.iter().map(PathBuf::as_path)
+    }
+
+    pub fn spec(&self) -> &TemplateSpec {
+        &self.spec
+    }
+}
+
 type Mapp = Map<String, MetaValue>;
 
 fn get_title(map: &Mapp) -> String {
@@ -132,21 +152,11 @@ fn get_bindings_filenames(map: &Mapp) -> Vec<PathBuf> {
     get_paths("", map, "bindings")
 }
 
-fn get_functions_filenames(map: &Mapp) -> Vec<PathBuf> {
-    get_paths("", map, "functions")
-}
-
-fn get_template_spec(map: &Mapp) -> Result<Option<(String, TemplateSpec)>> {
-    match get_string(map, "template") {
-        Some(s) => {
-            let mut spec_path = PathBuf::from(&s);
-            spec_path.push("template");
-            spec_path.push("template.yaml");
-            let spec = TemplateSpec::from_file(&spec_path)?;
-            Ok(Some((s, spec)))
-        }
-        None => Ok(None),
-    }
+fn load_template_spec(template: &str) -> Result<TemplateSpec> {
+    let mut spec_path = PathBuf::from(template);
+    spec_path.push("template");
+    spec_path.push("template.yaml");
+    TemplateSpec::from_file(&spec_path)
 }
 
 fn get_paths<P>(basedir: P, map: &Mapp, field: &str) -> Vec<PathBuf>
