@@ -1,7 +1,7 @@
 use std::path::{Component, Path, PathBuf};
 
+use log::trace;
 use pandoc::{InputFormat, InputKind, OutputFormat, OutputKind, Pandoc, PandocOption};
-use tracing::{event, instrument, Level};
 
 /// Get the base directory given the name of the markdown file.
 ///
@@ -22,7 +22,6 @@ pub fn get_basedir_from(filename: &Path) -> PathBuf {
 ///
 /// This attempts to determine if `--citeproc` or `--filter pandoc-citeproc`
 /// is needed, and then does that specific thing.
-#[instrument(skip(pandoc))]
 pub fn add_citeproc(pandoc: &mut Pandoc) {
     let mut guesser = Pandoc::new();
     guesser.set_input(InputKind::Pipe("".to_string()));
@@ -31,10 +30,10 @@ pub fn add_citeproc(pandoc: &mut Pandoc) {
     guesser.set_output(OutputKind::Pipe);
     guesser.add_option(PandocOption::Citeproc);
     if guesser.execute().is_ok() {
-        event!(Level::TRACE, "Discovered --citeproc");
+        trace!("Discovered --citeproc");
         pandoc.add_option(PandocOption::Citeproc);
     } else {
-        event!(Level::TRACE, "Discovered --filter pandoc-citeproc");
+        trace!("Discovered --filter pandoc-citeproc");
         pandoc.add_option(PandocOption::Filter("pandoc-citeproc".into()));
     }
 }
