@@ -5,6 +5,7 @@ use crate::ScenarioStep;
 use crate::StepKind;
 use crate::SubplotError;
 use crate::{DotMarkup, GraphMarkup, PikchrMarkup, PlantumlMarkup};
+use crate::{Warning, Warnings};
 
 use pandoc_ast::Attr;
 use pandoc_ast::Block;
@@ -13,7 +14,7 @@ use pandoc_ast::Target;
 
 /// Typeset an error as a Pandoc AST Block element.
 pub fn error(err: SubplotError) -> Block {
-    let msg = format!("ERROR: {}", err.to_string());
+    let msg = format!("ERROR: {}", err);
     Block::Para(error_msg(&msg))
 }
 
@@ -53,13 +54,13 @@ pub fn file_block(attr: &Attr, text: &str) -> Block {
 ///
 /// The snippet is given as a text string, which is parsed. It need
 /// not be a complete scenario, but it should consist of complete steps.
-pub fn scenario_snippet(bindings: &Bindings, snippet: &str) -> Block {
+pub fn scenario_snippet(bindings: &Bindings, snippet: &str, warnings: &mut Warnings) -> Block {
     let lines = parse_scenario_snippet(snippet);
     let mut steps = vec![];
     let mut prevkind: Option<StepKind> = None;
 
     for line in lines {
-        let (this, thiskind) = step(bindings, line, prevkind);
+        let (this, thiskind) = step(bindings, line, prevkind, warnings);
         steps.push(this);
         prevkind = thiskind;
     }
@@ -70,13 +71,14 @@ pub fn scenario_snippet(bindings: &Bindings, snippet: &str) -> Block {
 fn step(
     bindings: &Bindings,
     text: &str,
-    defkind: Option<StepKind>,
+    prevkind: Option<StepKind>,
+    warnings: &mut Warnings,
 ) -> (Vec<Inline>, Option<StepKind>) {
-    let step = ScenarioStep::new_from_str(text, defkind);
+    let step = ScenarioStep::new_from_str(text, prevkind);
     if step.is_err() {
         return (
             error_msg(&format!("Could not parse step: {}", text)),
-            defkind,
+            prevkind,
         );
     }
     let step = step.unwrap();
@@ -84,15 +86,13 @@ fn step(
     let m = match bindings.find("", &step) {
         Ok(m) => m,
         Err(e) => {
-            eprintln!("Could not select binding: {:?}", e);
-            return (
-                error_msg(&format!("Could not select binding for: {}", text)),
-                defkind,
-            );
+            let w = Warning::UnknownBinding(format!("{}", e));
+            warnings.push(w.clone());
+            return (error_msg(&format!("{}", w)), prevkind);
         }
     };
 
-    let mut inlines = vec![keyword(&step), space()];
+    let mut inlines = vec![keyword(&step, prevkind), space()];
 
     for part in m.parts() {
         match part {
@@ -106,9 +106,19 @@ fn step(
 
 // Typeset first word, which is assumed to be a keyword, of a scenario
 // step.
-fn keyword(step: &ScenarioStep) -> Inline {
-    let word = inlinestr(step.keyword());
-    Inline::Emph(vec![word])
+fn keyword(step: &ScenarioStep, prevkind: Option<StepKind>) -> Inline {
+    let actual = inlinestr(&format!("{}", step.kind()));
+    let and = inlinestr("and");
+    let keyword = if let Some(prevkind) = prevkind {
+        if prevkind == step.kind() {
+            and
+        } else {
+            actual
+        }
+    } else {
+        actual
+    };
+    Inline::Emph(vec![keyword])
 }
 
 // Typeset a space between words.
@@ -144,11 +154,11 @@ pub fn link_as_note(attr: Attr, text: Vec<Inline>, target: Target) -> Inline {
 ///
 /// If the code block which contained the pikchr contains other classes, they
 /// can be added to the SVG for use in later typesetting etc.
-pub fn pikchr_to_block(pikchr: &str, class: Option<&str>) -> Block {
+pub fn pikchr_to_block(pikchr: &str, class: Option<&str>, warnings: &mut Warnings) -> Block {
     match PikchrMarkup::new(pikchr, class).as_svg() {
         Ok(svg) => typeset_svg(svg),
         Err(err) => {
-            eprintln!("pikchr render failed: {}", err);
+            warnings.push(Warning::Pikchr(format!("{}", err)));
             error(err)
         }
     }
@@ -157,11 +167,11 @@ pub fn pikchr_to_block(pikchr: &str, class: Option<&str>) -> Block {
 // Take a dot graph, render it as SVG, and return an AST Block
 // element. The Block will contain the SVG data. This allows the graph
 // to be rendered without referending external entities.
-pub fn dot_to_block(dot: &str) -> Block {
+pub fn dot_to_block(dot: &str, warnings: &mut Warnings) -> Block {
     match DotMarkup::new(dot).as_svg() {
         Ok(svg) => typeset_svg(svg),
         Err(err) => {
-            eprintln!("dot failed: {}", err);
+            warnings.push(Warning::Dot(format!("{}", err)));
             error(err)
         }
     }
@@ -170,11 +180,11 @@ pub fn dot_to_block(dot: &str) -> Block {
 // Take a PlantUML graph, render it as SVG, and return an AST Block
 // element. The Block will contain the SVG data. This allows the graph
 // to be rendered without referending external entities.
-pub fn plantuml_to_block(markup: &str) -> Block {
+pub fn plantuml_to_block(markup: &str, warnings: &mut Warnings) -> Block {
     match PlantumlMarkup::new(markup).as_svg() {
         Ok(svg) => typeset_svg(svg),
         Err(err) => {
-            eprintln!("plantuml failed: {}", err);
+            warnings.push(Warning::Plantuml(format!("{}", err)));
             error(err)
         }
     }
@@ -182,13 +192,13 @@ pub fn plantuml_to_block(markup: &str) -> Block {
 
 /// Typeset a project roadmap expressed as textual YAML, and render it
 /// as an SVG image.
-pub fn roadmap_to_block(yaml: &str) -> Block {
+pub fn roadmap_to_block(yaml: &str, warnings: &mut Warnings) -> Block {
     match roadmap::from_yaml(yaml) {
         Ok(ref mut roadmap) => {
             roadmap.set_missing_statuses();
             let width = 50;
             match roadmap.format_as_dot(width) {
-                Ok(dot) => dot_to_block(&dot),
+                Ok(dot) => dot_to_block(&dot, warnings),
                 Err(e) => Block::Para(vec![inlinestr(&e.to_string())]),
             }
         }
